#ifndef __CURSOR_H_
#define __CURSOR_H_

#include "ocicpp.h"
#include "OraError.h"
#include "TableHead.h"
#include "RowCell.h"
#include "Lob.h"

const string default_null_text="<NULL>";
	
class Cursor {
private: /* OCI Defined handles */
	OCIEnv *envhp;
	OCISvcCtx *svchp;
	OCIStmt *stmthp;
	OCIParam *paramd;
	OCIError *errhp;
	OCIDefine **defineArr; /* Array of pointers to define descriptor */
private: 
	TableHead *head; /* Contains information about result set (colnames sizes types oracle and for inner use ).Array */
	RowCell *cell; /* Contains fetched row.Array */
	sb4 nCols;
	ub2 stmtType; /* Statement type as described in table 4-1 OCI Reference */
	sword status;
	string nulltext; /* Text for null fields */
	int haveResultSet; 
	int canFetch;
	map<string,int> cols_map; /* For get<Type>("colName") methods */
public:
	Cursor();
	Cursor(OCIEnv *,OCISvcCtx *,OCIError *er);
	void init(OCIEnv *,OCISvcCtx *,OCIError *er);
	~Cursor();
	void drop();
/* Once for request*/
	sb4 prepare(const string &);
	void exec();
	void describe();
	void define();
	void hashHead(); 
/* fetch returns non zero value if theres another rows */
	int fetch();
/* get<Type> methods working with hardcoded cols numbers */	
	void getStr(int col,string &) const;
	void getInt(int col,int &val) const;
	void getDouble(int col,double &val) const;
	void getCLOB(int col,Lob &) const;
	void getBLOB(int col,Lob &) const;
/* get<Type> same but working with attribute name */
	void getStr(const string &col,string &val) const;
	void getInt(const string &col,int &val) const;
	void getDouble(const string &col,double &val) const;
	void getCLOB(const string &col,Lob &lob) const;
	void getBLOB(const string &col,Lob &lob) const;
/* Some functions to describe select-list */	
	int getNCols() const;
	void getColName(int col,string &) const; 
	int getColSize(int col) const;
	int getColSize(const string &) const;
	void setNullText(const string &);
	int getColType(int col) const;
	int getColType(const string &) const;
/* Some supplementary functions */
	
private:			
	void correctTypes();
};
#endif