#include <ocicpp.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <string>
#include <map>
#include <iostream.h>
using namespace std;

void printHead(Cursor &);
void printRow(Cursor &);
void connect(Connection &);
void transStart(Connection &);
void transCommit(Connection &);
void transRollback(Connection &);
void version(Connection &);
void test_hash_cap(Connection &);
void exit(Connection &);
void test_blob(Connection &);

typedef void (*func_type) (Connection &);

map<string,func_type> commands_map;



int main(int argv,char **argc) {
	db::init();
	Connection con;
	// Cursor cur;
	string sql;
	
	commands_map["connect"]=connect;
	commands_map["transStart"]=transStart;
	commands_map["transCommit"]=transCommit;
	commands_map["transRollback"]=transRollback;
	commands_map["version"]=version;
	commands_map["test_hash_cap"]=test_hash_cap;
	commands_map["test_blob"]=test_blob;
	commands_map["exit"]=exit;
	while(1) {
		cout<<">";
		getline(cin,sql);
		try {
			 map<string,func_type>::const_iterator iter=commands_map.find(sql);
			 if(iter!=commands_map.end()) {
			 	(iter->second)(con);
			 } else {
			 	Cursor cur;
			 	cur.drop();
				con.execQuery(sql,cur);
				cout<<"executed ok"<<endl;
				printHead(cur);
				while(cur.fetch()) {
					printRow(cur);	
				}
			}
		} catch(OraError *er) {
			cout << "Catch exception" <<endl;
			cout << er->message<<endl;
		}
	}
}	




void connect(Connection &con) {
	string tns,user,passwd;
	cout << "connect string:";
	getline(cin,tns);
	cout<<"user:";
	getline(cin,user);
	cout<<"password:";
	getline(cin,passwd);
	cout<<"Connecting..."<<endl;
	con.drop();
	db::connect(tns.c_str(),user.c_str(),passwd.c_str(),con);
	cout<<"connected."<<endl;
}

void transStart(Connection &con) {
	con.transStart();
}

void transCommit(Connection &con) {
	con.transCommit();
}

void transRollback(Connection &con) {
	con.transRollback();
}

void version(Connection &con) {
	version();
}

void test_hash_cap(Connection &con) {
	Cursor cur;
	string sql,cell;
	cout << "sql:";
	getline(cin,sql);
	cout << "enter '<' when finish playing"<<endl;
	con.execQuery(sql,cur);
	printHead(cur);
	if(!cur.fetch()) return;
	cout<<"fetched ok"<<endl;
	while(1) {
		string field;
		cout<<"Field to print or '.' to next row:";
		cin>>field;
		if(field==".") {
			if(!cur.fetch()) break;
		} else if(field=="<") {
			break;	
		} else {
			try {
				cur.getStr(field,cell);
				cout<<cell<<endl;
			} catch(OraError *er) {
				if(er->errtype==OCICPPERROR) {
					cout<<er->message<<endl;
					continue;
				} else throw;
			}
		}
	}
}

void exit(Connection &con) {
	try {
		con.drop();
	} catch(OraError *er) {
		cout << er->message<<endl;
		exit(1);
	}
	exit(0);
}

void test_blob(Connection &con) {
	Cursor cur;
	string sql,field,mode,blob,dir,pos,len;
	cout << "sql: ";
	getline(cin,sql);
	con.execQuery(sql,cur);
	if(!cur.fetch()) {
		cout << "No more rows"<<endl;
		return;
	}
	while(1) {
		cout<< "field to process or . for next row:";
		getline(cin,field);
		if(field==".") {
			if(!cur.fetch()) {
				cout << "No more rows "<<endl;
				return;
			}
		} else if(field=="") {
			return;
		} else {
			Lob lob;
			unsigned actread;
			cur.getCLOB(field,lob);
			while(1) {
				cout << "mode(r/w/s/t/l/o) Enter to exit:";
				getline(cin,mode);
				if(mode=="s") {
					cout<<"Direction(SET/CUR/END:";
					getline(cin,dir);
					cout<<"Position:";
					getline(cin,pos);
					if(dir=="SET") lob.seek(atoi(pos.c_str()),SET);
					else if(dir=="CUR") lob.seek(atoi(pos.c_str()),CUR);
					else lob.seek(atoi(pos.c_str()),END);
				} else if(mode=="o") {
					cout<<"Offset:"<<lob.tell()<<endl;
				} else if(mode=="t") {
					cout << "new len:";
					getline(cin,len);
					lob.trunc(atoi(len.c_str()));
				} else if(mode=="l") {
					cout << "Len: "<<lob.getLen()<<endl;
				} else if(mode=="r") {
					char buf[256];
					while(actread=lob.read(buf,254)) {
						buf[actread]='\0';
						cout << buf;
					}
					cout << endl;
				} else if(mode=="w") {
					cout<<"data:";
					getline(cin,blob);
					lob.write(blob.c_str(),strlen(blob.c_str()));
				} else break;
			}
		}			
	}
}
	

void printHead(Cursor &cur) {
	string attrName;
	for(int i=0;i<cur.getNCols();i++) {
		cur.getColName(i,attrName);
		if(!i) cout<<"|";
		cout<<attrName<<"("<<cur.getColSize(i)<<")|";
	}
	cout << endl;
}

void printRow(Cursor &cur) {
	string cell;
	int nCols=cur.getNCols();
	for(int i=0;i<nCols;i++) {
		cur.getStr(i,cell);
		if(!i) cout<<"|";
		cout<<cell<<"|";
	}
	cout << endl;
}